==> relocation-0.3.0.tgz: package/src/connect.js <==
import {Component, createElement, PropTypes} from 'react';
import hoistStatics from 'hoist-non-react-statics';
import {connect} from 'react-redux';

import {getComponents} from './selector';
import {removeComponent, updateComponent} from './action';
import {componentsShape, renderMapShape, getDisplayName} from './util';

/**
 * Create a higher-order wrapper which provides an array of components to render
 * to its wrapped instance.
 *
 * @param {Object|Function} rawRenderMap An object with component type/render
 * function key value pairs or a function returning such an object.
 * @param {Object} defaultProps An object or a function returing such an
 * object.
 * @returns {Function} Higher-order component wrapper.
 */
export default ({scope, ...defaultProps} = {}) => (WrappedComponent) => {
  class Connect extends Component {
    static propTypes = {
      ___relocationDispatch___: PropTypes.shape({
        removeComponent: PropTypes.func.isRequired,
      }),
      ___relocationState___: PropTypes.shape({
        components: componentsShape.isRequired,
        renderMap: renderMapShape.isRequired,
      }).isRequired,
    };

    static contextTypes = {
      router: PropTypes.object,
    }

    render() {
      const {components, renderMap} = this.props.___relocationState___;
      const {
        removeComponent,
        updateComponent,
      } = this.props.___relocationDispatch___;

      const inRenderMap = (component) =>
        typeof renderMap[component.type] === 'function';

      const assign = (component) => {
        const result = {
          ...component,
          render: renderMap[component.type],
          update: (props) => updateComponent(component.id, props),
          remove: () => removeComponent(component.id),
        };

        if (scope) {
          result.scope = scope;
        }

        return result;
      };

      const currentComponents = components
        // Remove components not included in the render function map.
        .filter(inRenderMap)
        // Assign render update and remove functions and scope if it is defined.
        .map(assign);

      /* eslint-disable no-unused-vars */
      const {
        ___relocationState___,
        ___relocationDispatch___,
        ...childProps,
      } = this.props;
      /* eslint-enable no-unused-vars */

      const mergedProps = {
        ...childProps,
        ...scope
          ? {[scope]: {components: currentComponents}}
          : {components: currentComponents},
      };

      return <WrappedComponent {...mergedProps}/>;
    }
  }

  Connect.displayName = `Relocation(${getDisplayName(WrappedComponent)})`;

  const mapState = (state, props) => {
    const mergedProps = {
      ...defaultProps,
      ...scope ? props[scope] : props,
    };

    const {components, getRelocationState} = mergedProps;

    const selectorProps = getRelocationState
      ? {getRelocationState, ...props}
      : props;

    return {
      // Put everything in a ___relocationState___ namespace to avoid possible
      // conflict with existing props.
      ___relocationState___: {
        components: getComponents(state, selectorProps),
        renderMap: components,
      },
    };
  };

  const mapDispatch = (dispatch) => ({
      // Put everything in a ___relocationDispatch___ namespace to avoid
      // possible conflict with existing props.
    ___relocationDispatch___: {
      removeComponent: (id) => dispatch(removeComponent(id)),
      updateComponent: (id, props) => dispatch(updateComponent(id, props)),
    },
  });

  return connect(
    mapState,
    mapDispatch,
  )(hoistStatics(Connect, WrappedComponent));
};

==> relocation-0.4.0.tgz: package/src/connect.js <==
import {Component, createElement} from 'react';
import PropTypes from 'prop-types';
import hoistStatics from 'hoist-non-react-statics';
import {connect} from 'react-redux';

import {getComponents} from './selector';
import {removeComponent, updateComponent} from './action';
import {componentsShape, instancesShape, getDisplayName} from './util';

const getFinalComponents = (state, props) => {
  const initial = getComponents(state, props);
  const exists = (component) => {
    return typeof props.components[component.type] === 'function';
  };
  const shouldMount = props.componentShouldMount || (() => true);
  return initial.filter(
    (component) => exists(component) && shouldMount(state, props, component)
  );
};

/**
 * Create a higher-order wrapper which provides an array of components to render
 * to its wrapped instance.
 *
 * @param {Object|Function} rawRenderMap An object with component type/render
 * function key value pairs or a function returning such an object.
 * @param {Object} defaultProps An object or a function returing such an
 * object.
 * @returns {Function} Higher-order component wrapper.
 */
export default () => (WrappedComponent) => {
  class Connect extends Component {
    static propTypes = {
      ___relocationDispatch___: PropTypes.shape({
        removeComponent: PropTypes.func.isRequired,
        updateComponent: PropTypes.func.isRequired,
      }),
      ___relocationState___: PropTypes.shape({
        components: componentsShape.isRequired,
        instances: instancesShape.isRequired,
      }).isRequired,
    };

    render() {
      const {
        ___relocationState___,
        ___relocationDispatch___,
        ...childProps
      } = this.props;

      const {components, instances} = ___relocationState___;
      const {
        removeComponent,
        updateComponent,
      } = ___relocationDispatch___;

      const assign = (component) => {
        const result = {
          ...component,
          containerProps: childProps,
          render: components[component.type],
          update: (...args) => updateComponent(component.id, ...args),
          remove: (...args) => removeComponent(component.id, ...args),
        };

        return result;
      };

      const currentComponents = instances.map(assign);

      const mergedProps = {
        ...childProps,
        components: currentComponents,
      };

      return <WrappedComponent {...mergedProps}/>;
    }
  }

  Connect.displayName = `Relocation(${getDisplayName(WrappedComponent)})`;

  const mapState = (state, props) => {
    return {
      // Put everything in a ___relocationState___ namespace to avoid possible
      // conflict with existing props.
      ___relocationState___: {
        instances: getFinalComponents(state, props),
        components: props.components,
      },
    };
  };

  const mapDispatch = (dispatch) => ({
      // Put everything in a ___relocationDispatch___ namespace to avoid
      // possible conflict with existing props.
    ___relocationDispatch___: {
      removeComponent: (id) => dispatch(removeComponent(id)),
      updateComponent: (id, props) => dispatch(updateComponent(id, props)),
    },
  });

  return connect(
    mapState,
    mapDispatch,
  )(hoistStatics(Connect, WrappedComponent));
};
